import logging
import threading
from typing import List
import src.database.connection as database
from src.api.entities.plano_pagamento_cliente import PlanoPagamentoCliente
from src.database import queries
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
import src.database.utils as utils
#from src.entities.product import Product
#from src.entities.photos_sku import PhotosSku
import src.api.okvendas as api_okvendas
from src.jobs.system_jobs import OnlineLogger
from src.database.entities.client_payment_plan import ClientPaymentPlan
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_client_payment_plan(job_config_dict: dict):
    with lock:
        """
        Job para inserir os planos de pagamentos dos clientes no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir os planos de pagamentos dos clientes no semaforo nao encontrado', 'warning', 'CLIENT_PAYMENT_PLAN')
            return

        try:
            db_client_payment_plan = query_list_client_payment_plan(job_config_dict, db_config)
            if not insert_update_semaphore_client_payment_plan(job_config_dict, db_config, db_client_payment_plan):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Nao foi possivel inserir os planos de pagamentos dos clientes no banco semaforo', 'error', 'CLIENT_PAYMENT_PLAN')
                return

            split_client_payment_plan = dict.fromkeys(set([d.code_client for d in db_client_payment_plan]), [])
            for cpp in db_client_payment_plan:
                if cpp.code_client in cpp.code_client in split_client_payment_plan.keys():
                    if split_client_payment_plan[cpp.code_client] is not None and len(split_client_payment_plan[cpp.code_client]) <= 0:
                       split_client_payment_plan[cpp.code_client] = [cpp.payment_methods]
                    elif len(split_client_payment_plan[cpp.code_client]) > 0:
                       split_client_payment_plan[cpp.code_client].append(cpp.payment_methods)
                    else:
                       split_client_payment_plan[cpp.code_client] = []


            api_payment_plan_list = [PlanoPagamentoCliente(
                codigo_cliente=k,
                formas_pagamento=v) for k, v in split_client_payment_plan.items()]

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Enviando os planos de pagamentos dos clientes via api okvendas', 'info', 'CLIENT_PAYMENT_PLAN')

            for api_client_payment_plan in api_payment_plan_list:
                try:
                    response_list, status_code, lista = api_okvendas.post_client_payment_plan(api_client_payment_plan)
                    message_response = format_response(lista)
                    if not response_list:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao processar a forma de pagamento dos clientes do seguinte cliente {api_client_payment_plan.codigo_cliente} {message_response}', 'warning', 'CLIENT_PAYMENT_PLAN')

                    if status_code == 207:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                 f'Inserção/Atualização parcial das formas de pagamento dos clientes do seguinte cliente {api_client_payment_plan.codigo_cliente} {message_response}','warning', 'CLIENT_PAYMENT_PLAN')

                    if protocol_semaphore_client_payment_plan(job_config_dict, db_config, api_client_payment_plan.codigo_cliente):
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Plano de Pagamento do cliente {api_client_payment_plan.codigo_cliente} protocolado no banco semaforo', 'info', 'CLIENT_PAYMENT_PLAN')
                    else:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar o plano de pagamento do cliente {api_client_payment_plan.codigo_cliente} ', 'warning', 'CLIENT_PAYMENT_PLAN')

                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao enviar a forma de pagamento do cliente {api_client_payment_plan.codigo_cliente} : {str(e)}', 'error', 'CLIENT_PAYMENT_PLAN')

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro {str(ex)}', 'error', 'CLIENT_PAYMENT_PLAN')


def format_response(response):
    message = ''
    lista = response.get('Response')
    if lista is None:
        if response.get('Message') is not None:
            message = response.get('Message')
        return message

    for item in lista:
        if item['Identifiers'] is None or len(item['Identifiers']) <= 0:
            continue

        message = message + f'{item["Message"]} '

    return message


def query_list_client_payment_plan(job_config_dict: dict, db_config: DatabaseConfig) -> List[ClientPaymentPlan]:
    """
        Consultar no banco semáforo os planos de pagamentos dos clientes

        Args:
            job_config_dict: Configuração do job
            db_config: Configuração do banco de dados

        Returns:
        Lista dos planos de pagamentos dos clientes
        """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            lists = [ClientPaymentPlan(**p) for p in results]
            return lists

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar os planos de pagamentos dos clientes no banco semaforo: {str(ex)}', 'error', 'CLIENT_PAYMENT_PLAN')

    return []


def insert_update_semaphore_client_payment_plan(job_config_dict: dict, db_config: DatabaseConfig, lists: List[ClientPaymentPlan]) -> bool:
    """
    Insere os planos de pagamentos dos clientes no banco semáforo
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista dos planos de pagamentos dos clientes

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.code_client, ' ', IntegrationType.PLANO_PAGAMENTO_CLIENTE.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()

        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'{cursor.rowcount} clientes dos planos de pagamento foram inseridos no semaforo', 'info',
                 'CLIENT_PAYMENT_PLAN')

        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar os planos de pagamentos dos clientes no banco semaforo: {str(ex)}', 'error', 'CLIENT_PAYMENT_PLAN')

    return False


def protocol_semaphore_client_payment_plan(job_config_dict: dict, db_config: DatabaseConfig, identifier: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        if identifier is not None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar os planos de pagamentos no banco semaforo: {str(ex)}', 'error', 'CLIENT_PAYMENT_PLAN')



from typing import List, Dict
from datetime import datetime


class Address:
    zipcode: str
    phone: str
    description: str
    type: str
    address: str
    number: str
    complement: str
    reference: str
    neighbourhood: str
    city: str
    ibge_code: str
    state: str
    country: str
    id: int

    def __init__(self, cep: str, fone_contato: str, descricao_endereco: str, tipo_logradouro: str, endereco: str, numero: str, complemento: str, referencia_entrega: str, bairro: str, cidade: str, codigo_ibge: str,
                 estado: str, pais: str, id: int) -> None:
        self.zipcode = cep
        self.phone = fone_contato
        self.description = descricao_endereco
        self.type = tipo_logradouro
        self.address = endereco
        self.number = numero
        self.complement = complemento
        self.reference = referencia_entrega
        self.neighbourhood = bairro
        self.city = cidade
        self.ibge_code = codigo_ibge
        self.state = estado
        self.country = pais
        self.id = id


class ApprovedClient:
    id: int
    corporate_name: str
    person_type: str
    cpf: str
    cnpj: str
    municipal_registration: str
    state_registration: str
    mobile_phone: str
    home_phone: str
    erp_code: str
    deactivated_date: datetime
    buy_allowed: bool
    credit_limit: float
    requested_credit_limit: float
    activity_line: str
    representative_code: str
    segment: str
    login: str
    name: str
    sex: str
    email: str
    domain: str
    protocol: str
    addresses: List[Address]
    constitution_date: datetime
    updated_date: datetime
    register_date: datetime
    external_integration_date: datetime
    photo: str
    address_id: int

    def __init__(self, id: int, razao_social: str, tipo_pessoa: str, cpf: str, cnpj: str, inscricao_municipal: str, inscricao_estadual: str, fone_celular: str, fone_residencial: str,
                 codigo_referencia: str, data_desativacao: str, compra_liberada: bool, limite_credito: float, limite_credito_solicitado: float, ramo_atividade: str, codigo_representante: str,
                 segmento: str, login: str, nome: str, sexo: str, email: str, domain: str, protocolo: str, enderecos: List[Dict], data_constituicao: datetime, data_alteracao: str,
                 data_cadastro: str, data_integracao_externa: str, foto: str, endereco_id: int, **kwargs) -> None:
        self.id = id
        self.corporate_name = razao_social
        self.person_type = tipo_pessoa
        self.cpf = cpf
        self.cnpj = cnpj
        self.municipal_registration = inscricao_municipal
        self.state_registration = inscricao_estadual
        self.mobile_phone = fone_celular
        self.home_phone = fone_residencial
        self.erp_code = codigo_referencia
        self.deactivated_date = data_desativacao if data_desativacao is None else datetime.strptime(data_desativacao.replace('T', ' '), '%Y-%m-%d %H:%M:%S')
        self.buy_allowed = compra_liberada
        self.credit_limit = limite_credito
        self.requested_credit_limit = limite_credito_solicitado
        self.activity_line = ramo_atividade
        self.representative_code = codigo_representante
        self.segment = segmento
        self.login = login
        self.name = nome
        self.sex = sexo
        self.email = email
        self.domain = domain
        self.protocol = protocolo
        self.addresses = [Address(**e) for e in enderecos] if enderecos is not None else []
        self.constitution_date = data_constituicao
        self.updated_date = data_alteracao if data_alteracao is None else datetime.strptime(data_alteracao.replace('T', ' '), '%Y-%m-%d %H:%M:%S')
        self.register_date = data_cadastro if data_cadastro is None else datetime.strptime(data_cadastro.replace('T', ' '), '%Y-%m-%d %H:%M:%S')
        self.external_integration_date = data_integracao_externa if data_integracao_externa is None else datetime.strptime(data_integracao_externa.replace('T', ' '), '%Y-%m-%d %H:%M:%S')
        self.photo = foto
        self.address_id = endereco_id
        self.__dict__.update(kwargs)


class ApprovedClientResponse:
    total: int
    data: List[ApprovedClient]

    def __init__(self, total: int, data: List[Dict]) -> None:
        self.total = total
        self.data = [ApprovedClient(**d) for d in data] if data is not None else []



import logging
import threading
import time

import src.database.connection as database
import src.database.utils as utils
from src.jobs.client_payment_plan_jobs import query_list_client_payment_plan, insert_update_semaphore_client_payment_plan
from src.jobs.system_jobs import OnlineLogger
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log
CLIENT_PAYMENT_PLAN = 'CLIENT_PAYMENT_PLAN'


def job_insert_client_no_exists_payment_plan_semaphore(job_config_dict: dict):
    with lock:
        """
        Job para inserir os clientes dos planos de pagamento que não existem no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     'Comando sql para inserir os clientes dos planos de pagamento que não existem no banco semáforo nao foi encontrado', 'warning', CLIENT_PAYMENT_PLAN)
        else:
            db = database.Connection(db_config)
            conn = db.get_conect()
            cursor = conn.cursor()

            try:

                db_client_payment_plan = query_list_client_payment_plan(job_config_dict, db_config)

                if len(db_client_payment_plan) <= 0:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                             f'Não existem registros de clientes novos com planos de pagamento do lado do ERP, que não estejam cadastrados no banco semaforo \n {db_config.sql}', 'info', CLIENT_PAYMENT_PLAN)

                    return

                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'Inserindo clientes novos dos planos de pagamento do ERP, que ainda não existem no banco semaforo \n {db_config.sql}', 'info', CLIENT_PAYMENT_PLAN)

                if not insert_update_semaphore_client_payment_plan(job_config_dict, db_config, db_client_payment_plan):
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Nao foi possivel inserir os planos de pagamentos dos clientes novos no banco semaforo', 'error', 'CLIENT_PAYMENT_PLAN')
                    return

            except Exception as ex:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                         f'Erro ao inserir os clientes novos dos planos de pagamento, que ainda não existem no banco semaforo {str(ex)}', 'error', CLIENT_PAYMENT_PLAN)

            cursor.close()
            conn.commit()
            conn.close()



# Será criado uma lista
# # com leiaute da http://api.gmimportacao.openk.com.br/ok/swagger/ui/index#!/Catalogo/impostoNCM
class ImpostoProduto:
    def __init__(self, codigo_sku, ncm, grupo_tributacao, grupo_cliente,
                 uf_origem, uf_destino,mva_iva, icms_intraestadual,
                  icms_interestadual, percentual_reducao_base_calcul, identificador):
        self.codigo_sku: str = codigo_sku
        self.ncm: str = ncm
        self.grupo_tributacao: str = grupo_tributacao
        self.grupo_cliente: str = grupo_cliente
        self.uf_origem: str = uf_origem
        self.uf_destino: str = uf_destino
        self.mva_iva: float = mva_iva
        self.icms_intraestadual: float = icms_intraestadual
        self.icms_interestadual: str = icms_interestadual
        self.percentual_reducao_base_calculo: float = percentual_reducao_base_calcul
        self.identificador: str = identificador



# Será criado uma lista
# # com leiaute da http://api.gmimportacao.openk.com.br/ok/swagger/ui/index#!/Representative/representante

class RegiaoVenda:
    nome: str
    codigo_externo: str

    def __init__(self, rv_nome: str, rv_codigo_externo: str) -> None:
        self.nome = rv_nome
        self.codigo_externo = rv_codigo_externo

class Representante:
    def __init__(self, codigo_externo, nome, telefone_celular, login,
                 password, supervisor,ativar_representante, email, regiao_venda: RegiaoVenda):
        self.codigo_externo: str = codigo_externo
        self.nome: str = nome
        self.telefone_celular: str = telefone_celular
        self.login: str = login
        self.password: str = password
        self.supervisor: bool = supervisor
        self.ativar_representante: bool = ativar_representante
        self.email: str = email
        self.regiao_venda = regiao_venda





# Será criado uma lista
# # com leiaute da http://api.gmimportacao.openk.com.br/ok/swagger/ui/index#!/client/formapagamento
from typing import List
from datetime import datetime

class PlanoPagamentoCliente:
    def __init__(self, codigo_cliente, formas_pagamento):
        self.codigo_cliente: str = codigo_cliente
        self.formas_pagamento: List[str] = formas_pagamento





from enum import IntEnum


class IntegrationType(IntEnum):
    LISTA_PRECO = 1,
    PRECO = 2,
    ESTOQUE = 3,
    PRODUTO = 4,
    LISTA_PRECO_PRODUTO = 5,
    REPRESENTANTE = 6,
    IMPOSTO = 7,
    CLIENTE = 8,
    PLANO_PAGAMENTO_CLIENTE = 9


# Queries nao podem terminar com ponto e virgula

def get_product_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.produto set data_sincronizacao = now() where codigo_erp = %s and codigo_erp_sku = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.produto set data_sincronizacao = SYSDATE where codigo_erp = :1 and codigo_erp_sku = :2'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.produto set data_sincronizacao = getdate() where codigo_erp = ? and codigo_erp_sku = ?'


def get_stock_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.estoque_produto set data_sincronizacao = now() where codigo_erp = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.estoque_produto set data_sincronizacao = SYSDATE where codigo_erp = :codigo_erp'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.estoque_produto set data_sincronizacao = getdate() where codigo_erp = ?'


def get_price_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.preco_produto set data_sincronizacao = now() where codigo_erp = %s and preco_atual = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.preco_produto set data_sincronizacao = SYSDATE where codigo_erp = :1 and preco_atual = :2'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.preco_produto set data_sincronizacao = getdate() where codigo_erp = ? and preco_atual = ?'


def get_insert_client_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao, codigo_ibge)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.CLIENTE (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, 
														TIPO_LOGRADOURO, LOGRADOURO,NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO, REFERENCIA, DIRECAO, CODIGO_IBGE) 
				VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12, :13, :14, :15, :16, :17, :18) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao, codigo_ibge)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2b_client_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao, inscricao_estadual, codigo_ibge, cliente_erp)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.CLIENTE (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, 
														TIPO_LOGRADOURO, LOGRADOURO,NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO, REFERENCIA, DIRECAO, INSCRICAO_ESTADUAL, CODIGO_IBGE, CLIENTE_ERP) 
				VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12, :13, :14, :15, :16, :17, :18, :19, :20) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao, inscricao_estadual, codigo_ibge, cliente_erp)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2c_client_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.CLIENTE (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, 
														TIPO_LOGRADOURO, LOGRADOURO,NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO, REFERENCIA, DIRECAO) 
				VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12, :13, :14, :15, :16, :17) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia, direcao)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_order_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, loja_id, opcao_pagamento_erp, end_entrega_cep, end_entrega_tipo_logradouro, 
					end_entrega_logradouro, end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, 
					end_entrega_estado, end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.PEDIDO (PEDIDO_ID, PEDIDO_VENDA_ID, DATA_PEDIDO, STATUS, CLIENTE_ID, VALOR, VALOR_DESCONTO, VALOR_FRETE, 
					VALOR_ADICIONAL, DATA_PAGAMENTO, TIPO_PAGAMENTO, BANDEIRA, PARCELAS, CONDICAO_PAGAMENTO_ERP, CODIGO_RASTREIO, DATA_PREVISAO_ENTREGA, 
					TRANSPORTADORA, MODO_ENVIO, CANAL_ID, LOJA_ID, OPCAO_PAGAMENTO_ERP, END_ENTREGA_CEP, END_ENTREGA_TIPO_LOGRADOURO, 
					END_ENTREGA_LOGRADOURO, END_ENTREGA_NUMERO, END_ENTREGA_COMPLEMENTO, END_ENTREGA_BAIRRO, END_ENTREGA_CIDADE, 
					END_ENTREGA_ESTADO, END_ENTREGA_REFERENCIA_ENT, END_ENTREGA_CODIGO_IBGE)
					VALUES (:1, :2, TO_DATE(:3, 'YYYY-MM-DD HH24:MI:SS'), :4, :5, :6, :7, :8, :9, TO_DATE(:10, 'YYYY-MM-DD HH24:MI:SS'), :11, :12, :13, :14, :15, TO_DATE(:16, 'YYYY-MM-DD HH24:MI:SS'), :17, :18, :19, :20, :21, :22, :23, :24, :25, :26, :27, :28, :29, :30, :31) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, loja_id, opcao_pagamento_erp, end_entrega_cep, end_entrega_tipo_logradouro, 
					end_entrega_logradouro, end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, 
					end_entrega_estado, end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2c_order_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, loja_id, end_entrega_cep, end_entrega_tipo_logradouro, end_entrega_logradouro, 
					end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, end_entrega_estado, 
					end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.PEDIDO (PEDIDO_ID, PEDIDO_VENDA_ID, DATA_PEDIDO, STATUS, CLIENTE_ID, VALOR, VALOR_DESCONTO, VALOR_FRETE, 
					VALOR_ADICIONAL, DATA_PAGAMENTO, TIPO_PAGAMENTO, BANDEIRA, PARCELAS, CONDICAO_PAGAMENTO_ERP, CODIGO_RASTREIO, DATA_PREVISAO_ENTREGA, 
					TRANSPORTADORA, MODO_ENVIO, CANAL_ID, LOJA_ID, END_ENTREGA_CEP, END_ENTREGA_TIPO_LOGRADOURO, END_ENTREGA_LOGRADOURO, 
					END_ENTREGA_NUMERO, END_ENTREGA_COMPLEMENTO, END_ENTREGA_BAIRRO, END_ENTREGA_CIDADE, END_ENTREGA_ESTADO, 
					END_ENTREGA_REFERENCIA_ENT, END_ENTREGA_CODIGO_IBGE)
					VALUES (:1, :2, TO_DATE(:3, 'YYYY-MM-DD HH24:MI:SS'), :4, :5, :6, :7, :8, :9, TO_DATE(:10, 'YYYY-MM-DD HH24:MI:SS'), :11, :12, :13, :14, :15, TO_DATE(:16, 'YYYY-MM-DD HH24:MI:SS'), :17, :18, :19, :20, :21, :22, :23, :24, :25, :26, :27, :28, :29, :30) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, loja_id, end_entrega_cep, end_entrega_tipo_logradouro, 
					end_entrega_logradouro, end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, 
					end_entrega_estado, end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2b_order_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, representante_erp, loja_id, opcao_pagamento_erp, cnpj_intermediador, codigo_pedido_canal, end_entrega_cep, end_entrega_tipo_logradouro, end_entrega_logradouro, end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, end_entrega_estado, end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.PEDIDO (PEDIDO_ID, PEDIDO_VENDA_ID, DATA_PEDIDO, STATUS, CLIENTE_ID, VALOR, VALOR_DESCONTO, VALOR_FRETE, 
					VALOR_ADICIONAL, DATA_PAGAMENTO, TIPO_PAGAMENTO, BANDEIRA, PARCELAS, CONDICAO_PAGAMENTO_ERP, CODIGO_RASTREIO, DATA_PREVISAO_ENTREGA, 
					TRANSPORTADORA, MODO_ENVIO, CANAL_ID, REPRESENTANTE_ERP, LOJA_ID, OPCAO_PAGAMENTO_ERP, CNPJ_INTERMEDIADOR, CODIGO_PEDIDO_CANAL, END_ENTREGA_CEP, END_ENTREGA_TIPO_LOGRADOURO, END_ENTREGA_LOGRADOURO, END_ENTREGA_NUMERO, END_ENTREGA_COMPLEMENTO, END_ENTREGA_BAIRRO, END_ENTREGA_CIDADE, END_ENTREGA_ESTADO, END_ENTREGA_REFERENCIA_ENT, END_ENTREGA_CODIGO_IBGE)
					VALUES (:1, :2, TO_DATE(:3, 'YYYY-MM-DD HH24:MI:SS'), :4, :5, :6, :7, :8, :9, TO_DATE(:10, 'YYYY-MM-DD HH24:MI:SS'), :11, :12, :13, :14, :15, TO_DATE(:16, 'YYYY-MM-DD HH24:MI:SS'), :17, :18, :19, :20, :21, :22, :23, :24, :25, :26, :27, :28, :29, :30, :31, :32, :33, :34 ) '''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio, canal_id, representante_erp, loja_id, opcao_pagamento_erp, cnpj_intermediador, codigo_pedido_canal, end_entrega_cep, end_entrega_tipo_logradouro, end_entrega_logradouro, end_entrega_numero, end_entrega_complemento, end_entrega_bairro, end_entrega_cidade, end_entrega_estado, end_entrega_referencia_ent, end_entrega_codigo_ibge)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_update_b2b_order_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.pedido
					SET condicao_pagamento_erp = %s
					  , transportadora = %s
					  , modo_envio = %s
					  , canal_id = %s
					  , representante_erp = %s
					  , opcao_pagamento_erp = %s
					  , cnpj_intermediador = %s
					  , codigo_pedido_canal = %s
					  , end_entrega_cep = %s
					  , end_entrega_tipo_logradouro = %s
					  , end_entrega_logradouro = %s
					  , end_entrega_numero = %s
					  , end_entrega_complemento = %s
					  , end_entrega_bairro = %s
					  , end_entrega_cidade = %s
					  , end_entrega_estado = %s
					  , end_entrega_referencia_ent = %s
					  , end_entrega_codigo_ibge = %s
					WHERE pedido_id = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.PEDIDO
					SET CONDICAO_PAGAMENTO_ERP = :1
					  , TRANSPORTADORA = :2
					  , MODO_ENVIO = :3
					  , CANAL_ID = :4
					  , REPRESENTANTE_ERP = :5
					  , OPCAO_PAGAMENTO_ERP = :6
					  , CNPJ_INTERMEDIADOR = :7
					  , CODIGO_PEDIDO_CANAL = :8
					  , END_ENTREGA_CEP = :9
					  , END_ENTREGA_TIPO_LOGRADOURO = :10
					  , END_ENTREGA_LOGRADOURO = :11
					  , END_ENTREGA_NUMERO = :12
					  , END_ENTREGA_COMPLEMENTO = :13
					  , END_ENTREGA_BAIRRO = :14
					  , END_ENTREGA_CIDADE = :15
					  , END_ENTREGA_ESTADO = :16
					  , END_ENTREGA_REFERENCIA_ENT = :17
					  , END_ENTREGA_CODIGO_IBGE = :18
					WHERE PEDIDO_ID = :19 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.pedido
					SET condicao_pagamento_erp = ?
					  , transportadora = ?
					  , modo_envio = ?
					  , canal_id = ?
					  , representante_erp = ?
					  , opcao_pagamento_erp = ?
					  , cnpj_intermediador = ?
					  , codigo_pedido_canal = ?
					  , end_entrega_cep = ?
					  , end_entrega_tipo_logradouro = ?
					  , end_entrega_logradouro = ?
					  , end_entrega_numero = ?
					  , end_entrega_complemento = ?
					  , end_entrega_bairro = ?
					  , end_entrega_cidade = ?
					  , end_entrega_estado = ?
					  , end_entrega_referencia_ent = ?
					  , end_entrega_codigo_ibge = ?
					WHERE pedido_id = ? '''


def get_insert_order_items_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (%s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.ITENS_PEDIDO (PEDIDO_ID, SKU, CODIGO_ERP, QUANTIDADE, EAN, VALOR, VALOR_DESCONTO, VALOR_FRETE)
					VALUES (:1, :2, :3, :4, :5, :6, :7, :8)'''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2c_order_items_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (%s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.ITENS_PEDIDO (PEDIDO_ID, SKU, CODIGO_ERP, QUANTIDADE, EAN, VALOR, VALOR_DESCONTO, VALOR_FRETE)
					VALUES (:1, :2, :3, :4, :5, :6, :7, :8)'''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_b2b_order_items_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete, cnpj_filial_venda, codigo_filial_erp, codigo_filial_expedicao_erp, codigo_filial_faturamento_erp) 
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.ITENS_PEDIDO (PEDIDO_ID, SKU, CODIGO_ERP, QUANTIDADE, EAN, VALOR, VALOR_DESCONTO, VALOR_FRETE, CNPJ_FILIAL_VENDA, CODIGO_FILIAL_ERP, CODIGO_FILIAL_EXPEDICAO_ERP, CODIGO_FILIAL_FATURAMENTO_ERP)
					VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12)'''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete, cnpj_filial_venda, codigo_filial_erp, codigo_filial_expedicao_erp, codigo_filial_faturamento_erp) 
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_query_client_id(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.cliente where email = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE EMAIL = :email'
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.cliente where email = ?'


def get_query_client_erp(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.cliente where cliente_erp = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE CLIENTE_ERP = :cliente_erp '
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.cliente where client_erp = ?'


def get_query_client_cpf(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.cliente where cpf = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE CPF = :cpf '
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.cliente where cpf = ?'


def get_query_client_integrado_cpf(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id, cliente_erp from openk_semaforo.cliente where cpf = %s and cliente_erp is not null'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID, cliente_erp FROM OPENK_SEMAFORO.CLIENTE WHERE CPF = :cpf and cliente_erp is not null'
    elif connection_type.lower() == 'sql':
        return 'select id, cliente_erp from openk_semaforo.cliente where cpf = ? and cliente_erp is not null'


def get_query_client_cnpj(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.cliente where cnpj = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE CNPJ = :cnpj '
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.cliente where cnpj = ?'


def get_query_client_integrado_cnpj(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id, cliente_erp from openk_semaforo.cliente where cnpj = %s and cliente_erp is not null'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID, cliente_erp FROM OPENK_SEMAFORO.CLIENTE WHERE CNPJ = :cnpj and cliente_erp is not null'
    elif connection_type.lower() == 'sql':
        return 'select id, cliente_erp from openk_semaforo.cliente where cnpj = ? and cliente_erp is not null'


def get_insert_out_clients(connection_type: str) -> str:
    if connection_type.lower() == 'mysql':
        return '''insert into openk_semaforo.cliente (cliente_erp, data_alteracao, data_sincronizacao)
					values (%s, %s, %s) '''
    elif connection_type.lower() == 'oracle':
        return '''INSERT INTO OPENK_SEMAFORO.CLIENTE (CLIENTE_ERP, DATA_ALTERACAO, DATA_SINCRONIZACAO)
					VALUES (:1, :2, :3)'''
    elif connection_type.lower() == 'sql':
        return '''insert into openk_semaforo.cliente (cliente_erp, data_alteracao, data_sincronizacao)
					values (?, ?, ?) '''


def get_update_client_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					WHERE cliente_erp = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					WHERE CLIENTE_ERP = :18 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							WHERE cliente_erp = ? '''

    def get_update_client_sql(connection_type: str):
        if connection_type.lower() == 'mysql':
            return '''update openk_semaforo.cliente
						SET nome = %s
						  , razao_social = %s
						  , cpf = %s
						  , cnpj = %s
						  , email = %s
						  , telefone_residencial = %s
						  , telefone_celular = %s
						  , cep = %s
						  , tipo_logradouro = %s
						  , logradouro = %s
						  , numero = %s
						  , complemento = %s
						  , bairro = %s
						  , cidade = %s
						  , estado = %s
						  , referencia = %s
						  , codigo_ibge = %s
						WHERE cliente_erp = %s '''
        elif connection_type.lower() == 'oracle':
            return '''UPDATE OPENK_SEMAFORO.CLIENTE
						SET NOME = :1
						  , RAZAO_SOCIAL = :2
						  , CPF = :3
						  , CNPJ = :4
						  , EMAIL = :5
						  , TELEFONE_RESIDENCIAL = :6
						  , TELEFONE_CELULAR = :7
						  , CEP = :8
						  , TIPO_LOGRADOURO = :9
						  , LOGRADOURO = :10
						  , NUMERO = :11
						  , COMPLEMENTO = :12
						  , BAIRRO = :13
						  , CIDADE = :14
						  , ESTADO = :15
						  , REFERENCIA = :16
						  , CODIGO_IBGE = :17
						WHERE CLIENTE_ERP = :18 '''
        elif connection_type.lower() == 'sql':
            return '''update openk_semaforo.cliente
								SET nome = ?
								  , razao_social = ?
								  , cpf = ?
								  , cnpj = ?
								  , email = ?
								  , telefone_residencial = ?
								  , telefone_celular = ?
								  , cep = ?
								  , tipo_logradouro = ?
								  , logradouro = ?
								  , numero = ?
								  , complemento = ?
								  , bairro = ?
								  , cidade = ?
								  , estado = ?
								  , referencia = ?
								  , codigo_ibge = ?
								WHERE cliente_erp = ? '''


def get_update_client_cpf_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					WHERE cpf = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					WHERE CPF = :18 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							WHERE cpf = ? '''


def get_update_client_cnpj_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					WHERE cnpj = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					WHERE CNPJ = :18 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							WHERE cnpj = ? '''


def get_update_b2b_client_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					  , inscricao_estadual = %s
					WHERE cliente_erp = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					  , INSCRICAO_ESTADUAL = :18
					WHERE CLIENTE_ERP = :19 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							  , inscricao_estadual = ?
							WHERE cliente_erp = ? '''


def get_update_b2b_client_cpf_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					  , inscricao_estadual = %s
					WHERE cpf = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					  , INSCRICAO_ESTADUAL = :18
					WHERE CPF = :19 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							  , inscricao_estadual = ?
							WHERE cpf = ? '''


def get_update_b2b_client_cnpj_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					  , codigo_ibge = %s
					  , inscricao_estadual = %s
					WHERE cnpj = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					  , CODIGO_IBGE = :17
					  , INSCRICAO_ESTADUAL = :18
					WHERE CNPJ = :19 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							  , codigo_ibge = ?
							  , inscricao_estadual = ?
							WHERE cnpj = ? '''


def get_update_b2c_client_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					WHERE cliente_erp = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					WHERE CLIENTE_ERP = :17 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							WHERE cliente_erp = ? '''


def get_update_b2c_client_cpf_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					WHERE cpf = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					WHERE CPF = :17 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							WHERE cpf = ? '''


def get_update_b2c_client_cnpj_sql(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					WHERE cnpj = %s '''
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					WHERE CNPJ = :17 '''
    elif connection_type.lower() == 'sql':
        return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							WHERE cnpj = ? '''


def get_query_non_integrated_order(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.pedido where pedido_id = %s and data_sincronizacao is null'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :order_id AND DATA_SINCRONIZACAO IS NULL'
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.pedido where pedido_id = ? and data_sincronizacao is null'


def get_query_order(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select id from openk_semaforo.pedido where pedido_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT ID FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :order_id'
    elif connection_type.lower() == 'sql':
        return 'select id from openk_semaforo.pedido where pedido_id = ?'


def get_query_order_status(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select status from openk_semaforo.pedido where pedido_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT STATUS FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :order_id'
    elif connection_type.lower() == 'sql':
        return 'select status from openk_semaforo.pedido where pedido_id = ?'


def get_query_order_sync_date(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'select if(data_sincronizacao is not null, true, false) from openk_semaforo.pedido where pedido_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'SELECT CASE WHEN DATA_SINCRONIZACAO IS NULL THEN 0 ELSE 1 END FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :order_id'
    elif connection_type.lower() == 'sql':
        return 'select if(data_sincronizacao is not null, true, false) from openk_semaforo.pedido where pedido_id = ?'


def get_command_parameter(connection_type: str, parameters: list):
    if connection_type.lower() == 'mysql':
        return tuple(parameters)
    elif connection_type.lower() == 'oracle':
        return parameters
    elif connection_type.lower() == 'sql':
        return parameters


def get_order_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.pedido set data_sincronizacao = now(), codigo_referencia = %s where pedido_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'UPDATE OPENK_SEMAFORO.PEDIDO SET DATA_SINCRONIZACAO = SYSDATE, CODIGO_REFERENCIA = :order_erp_id WHERE PEDIDO_ID = :order_id'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.pedido set data_sincronizacao = getdate(), codigo_referencia = ? where pedido_id = ?'


def get_order_protocol_nf_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.pedido set data_sincronizacao_nf = now() where pedido_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'UPDATE OPENK_SEMAFORO.PEDIDO SET DATA_SINCRONIZACAO_NF = SYSDATE WHERE PEDIDO_ID = :order_id'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.pedido set data_sincronizacao_nf = getdate() where pedido_id = ?'


def get_client_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.cliente set cliente_erp = %s where id = (select cliente_id from openk_semaforo.pedido where pedido_id = %s)'
    elif connection_type.lower() == 'oracle':
        return '''UPDATE OPENK_SEMAFORO.CLIENTE C
				SET CLIENTE_ERP = :1 
				WHERE ID = (SELECT CLIENTE_ID FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :2)'''
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.cliente set cliente_erp = ? where id = (select cliente_id from openk_semaforo.pedido where pedido_id = ?)'


def get_out_client_protocol_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.cliente set data_sincronizacao = now() where cliente_erp = %s'
    elif connection_type.lower() == 'oracle':
        return 'UPDATE OPENK_SEMAFORO.CLIENTE C SET DATA_SINCRONIZACAO = SYSDATE WHERE CLIENTE_ERP = :1 '
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.cliente set data_sincronizacao = getdate() where cliente_erp = ? '


def get_insert_update_semaphore_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'insert into openk_semaforo.semaforo(identificador, identificador2, tipo_id, data_alteracao, data_sincronizacao) values (%s, %s, %s, now(), null) on duplicate key update data_alteracao = now()'
    elif connection_type.lower() == 'oracle':
        return '''
		MERGE INTO OPENK_SEMAFORO.SEMAFORO sem
		USING (
			SELECT
				:1 AS identificador,
				:2 AS identificador2,
				:3 AS tipo_id
			FROM DUAL
		) tmp ON (tmp.identificador = sem.identificador AND tmp.identificador2 = sem.identificador2 AND tmp.tipo_id = sem.tipo_id)
		
		WHEN MATCHED THEN
			UPDATE SET sem.DATA_ALTERACAO = SYSDATE
		
		WHEN NOT MATCHED THEN  
		INSERT (IDENTIFICADOR, IDENTIFICADOR2, TIPO_ID, DATA_ALTERACAO, DATA_SINCRONIZACAO)
		VALUES (tmp.identificador, tmp.identificador2, tmp.tipo_id, sysdate, null)
		'''
    elif connection_type.lower() == 'sql':
        return '''
		MERGE INTO OPENK_SEMAFORO.SEMAFORO sem
		USING (
			SELECT
				? AS identificador,
				? AS identificador2,
				? AS tipo_id
		) tmp ON (tmp.identificador = sem.identificador AND tmp.identificador2 = sem.identificador2 AND tmp.tipo_id = sem.tipo_id)
		
		WHEN MATCHED THEN
			UPDATE SET sem.DATA_ALTERACAO = getdate()
		
		WHEN NOT MATCHED THEN
		INSERT (IDENTIFICADOR, IDENTIFICADOR2, TIPO_ID, DATA_ALTERACAO, DATA_SINCRONIZACAO)
		VALUES (tmp.identificador, tmp.identificador2, tmp.tipo_id, getdate(), null)
		'''


def get_protocol_semaphore_id_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = now() where identificador = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.semaforo set data_sincronizacao = sysdate where identificador = :1'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = getdate() where identificador = ?'

def get_protocol_semaphore_id2_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = now() where identificador = %s and identificador2 = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.semaforo set data_sincronizacao = sysdate where identificador = :1 and identificador2 = :2'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = getdate() where identificador = ? and identificador2 = ?'

def get_protocol_semaphore_id3_command(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = now() where identificador = %s and tipo_id = %s'
    elif connection_type.lower() == 'oracle':
        return 'update openk_semaforo.semaforo set data_sincronizacao = sysdate where identificador = :1 and tipo_id = :2'
    elif connection_type.lower() == 'sql':
        return 'update openk_semaforo.semaforo set data_sincronizacao = getdate() where identificador = ? and tipo_id = ?'

def get_insert_in_clients(connection_type: str):
    if connection_type.lower() == 'mysql':
        return '''
			insert into cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia,
								cliente_erp, direcao, data_sincronizacao, data_alteracao, codigo_ibge, inscricaoestadual)
		    value (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, null, %s, %s, null, %s, %s)
		    on duplicate key update
		    	data_alteracao = values(data_alteracao),
		    	data_sincronizacao = null
		'''
    elif connection_type.lower() == 'oracle':
        return '''
			MERGE INTO OPENK_SEMAFORO.CLIENTE c
			USING (
				SELECT
					:1 AS CPF,
					:2 AS CNPJ,
					:3 AS DATA_ALTERACAO
				FROM DUAL
			) tmp ON (tmp.CPF = c.CPF OR tmp.CNPJ = c.CNPJ)
			
			WHEN MATCHED THEN
				UPDATE SET
					c.DATA_ALTERACAO = tmp.DATA_ALTERACAO,
					c.DATA_SINCRONIZACAO = NULL
			
			WHEN NOT MATCHED THEN
			INSERT (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, TIPO_LOGRADOURO, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO,
					REFERENCIA, CLIENTE_ERP, DIRECAO, DATA_ALTERACAO, DATA_SINCRONIZACAO, CODIGO_IBGE, INSCRICAO_ESTADUAL)
			VALUES (:4, :5, :6, :7, :8, :9, :10, :11, :12, :13, :14, :15, :16, :17, :18, :19, NULL, :20, :21, NULL, :22, :23)
		'''
    elif connection_type.lower() == 'sql':
        return '''
			MERGE INTO OPENK_SEMAFORO.CLIENTE c
			USING (
				SELECT
					? AS NOME,
					? AS RAZAO_SOCIAL,
					? AS CPF,
					? AS CNPJ,
					? AS EMAIL,
					? AS TELEFONE_RESIDENCIAL,
					? AS TELEFONE_CELULAR,
					? AS CEP,
					? AS TIPO_LOGRADOURO,
					? AS LOGRADOURO,
					? AS NUMERO,
					? AS COMPLEMENTO,
					? AS BAIRRO,
					? AS CIDADE,
					? AS ESTADO,
					? AS REFERENCIA,
					? AS DIRECAO,
					? AS DATA_ALTERACAO,
					? AS CODIGO_IBGE,
					? AS INSCRICAO_ESTADUAL,
					? AS CLIENTE_ERP
				FROM DUAL
			) tmp ON (tmp.CPF = c.CPF OR tmp.CNPJ = c.CNPJ)
			
			WHEN MATCHED THEN
				UPDATE SET
					c.DATA_ALTERACAO = tmp.DATA_ALTERACAO,
					c.DATA_SINCRONIZACAO = NULL
			
			WHEN NOT MATCHED THEN
			INSERT (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, TIPO_LOGRADOURO, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO,
					REFERENCIA, CLIENTE_ERP, DIRECAO, DATA_ALTERACAO, DATA_SINCRONIZACAO, CODIGO_IBGE, INSCRICAO_ESTADUAL)
			VALUES (tmp.NOME, tmp.RAZAO_SOCIAL, tmp.CPF, tmp.CNPJ, tmp.EMAIL, tmp.TELEFONE_RESIDENCIAL, tmp.TELEFONE_CELULAR, tmp.CEP, tmp.TIPO_LOGRADOURO, tmp.LOGRADOURO, tmp.NUMERO, tmp.COMPLEMENTO,
					tmp.BAIRRO, tmp.CIDADE, tmp.ESTADO, tmp.REFERENCIA, NULL, tmp.DIRECAO, tmp.DATA_ALTERACAO, NULL, tmp.CODIGO_IBGE, tmp.INSCRICAO_ESTADUAL)
		'''


def get_update_in_clients(connection_type: str):
    if connection_type.lower() == 'mysql':
        return 'update openk_semaforo.cliente set data_alteracao = %s, data_sincronizacao = null where id = %s'
    elif connection_type.lower() == 'oracle':
        return 'UPDATE OPENK_SEMAFORO.CLIENTE SET DATA_ALTERACAO = :1, DATA_SINCRONIZACAO = NULL WHERE id = :2'
    elif connection_type.lower() == 'sql':
        return 'UPDATE OPENK_SEMAFORO.CLIENTE SET DATA_ALTERACAO = ?, DATA_SINCRONIZACAO = NULL WHERE id = ?'



import csv
import logging
import threading
from typing import List
import src
import src.database.connection as database
import pandas as pd
from src.api.entities.imposto_produto import ImpostoProduto
from src.database import queries
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
import src.database.utils as utils
from src.entities.product import Product
from src.entities.photos_sku import PhotosSku
import src.api.okvendas as api_okvendas
from src.jobs.system_jobs import OnlineLogger
from src.database.entities.product_tax import ProductTax
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_update_products_semaphore(job_config_dict: dict):
    with lock:
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        try:
            db_config = utils.get_database_config(job_config_dict)
            if db_config.sql is None:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para criar produtos nao encontrado', 'warning', 'PRODUTO')
                return

            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()

            cursor.execute(db_config.sql)
            cursor.close()

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produtos marcados para atualizar no banco semaforo: {cursor.rowcount}', 'info', 'PRODUTO')
            connection.commit()
            connection.close()

        except Exception as e:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao atualizar produtos no banco semaforo: {str(e)}', 'error', 'PRODUTO')


def job_insert_products_semaphore(job_config_dict: dict):
    with lock:
        """
        Job que realiza a insercao dos produtos no banco semaforo
        Args:
            job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        try:
            db_config = utils.get_database_config(job_config_dict)
            if db_config.sql is None:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir produtos no semaforo nao encontrado', 'warning', 'PRODUTO')
                return

            # abre a connection com o banco
            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()

            try:
                # sql: str = db_config.sql
                # if db_config.sql[len(db_config.sql) - 1] is ';':
                #     sql = db_config.sql[len(db_config.sql) - 1] = ' '

                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Inserindo produtos no banco semáforo', 'info', 'PRODUTO')
                cursor.execute(db_config.sql)
                connection.commit()
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{cursor.rowcount} produtos inseridos no banco semáforo', 'info', 'PRODUTO')
            except Exception as e:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao inserir produtos no banco semaforo: {str(e)}', 'error', 'PRODUTO')
                cursor.close()
                connection.close()

            cursor.close()
            connection.close()
        except Exception as e:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro ao durante a execucao do job: {str(e)}', 'error', 'PRODUTO')


def job_send_products(job_config_dict: dict):
    with lock:
        """
        Job que realiza a leitura dos produtos contidos no banco semaforo e envia para a api okvendas
        Args:
            job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        try:
            db_config = utils.get_database_config(job_config_dict)
            if db_config.sql is None:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para criar produtos nao encontrado', 'warning', 'PRODUTO')
                return

            produtos_repetidos = query_products(db_config)

            keys_photos_sent = {}
            photos_sku = mount_list_photos(produtos_repetidos)
            produtos = remove_repeat_products(produtos_repetidos)

            if len(produtos) <= 0:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Nao existem produtos para criar no momento', 'warning', 'PRODUTO')
                return

            for prod in produtos:
                try:
                    response = api_okvendas.post_produtos([prod])
                    if src.print_payloads:
                        print(response)

                    if response is not None and len(response) > 0:
                        for res in response:
                            if res.status > 1:
                                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                         f'Erro ao gerar produto {res.codigo_erp} na api okvendas. Erro gerado na api: {res.message}', 'warning', 'PRODUTO')
                            else:
                                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Produto {res.codigo_erp} criado com sucesso', 'info', 'PRODUTO')
                                if src.print_payloads:
                                    print(f'prod.codigo_erp = {prod.codigo_erp}, prod.preco_estoque[0].codigo_erp_atributo = {prod.preco_estoque[0].codigo_erp_atributo} ')
                                protocol_products(job_config_dict, [prod], db_config)

                                keys_photos_sent[(prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo)] = ""

                    else:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro no envio do produto codigo_erp: {prod.codigo_erp} codigo_referencia: {prod.codigo_referencia} Exceção: {str(e)}', 'error', 'PRODUTO')

                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro durante o envio de produtos: {str(e)}', 'error', 'PRODUTO')

            if send_photos_products(photos_sku, keys_photos_sent, 50):
                logger.info(job_config_dict.get('job_name') + f' | Fotos dos produtos cadastradas com sucesso')
            else:
                print(" Erro ao cadastrar as fotos do produto")
                logger.error(job_config_dict.get('job_name') + f' | Erro durante o cadastro das fotos dos produtos')

        except Exception as e:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro durante a execucao do job: {str(e)}', 'error', 'PRODUTO')


def query_products(db_config: DatabaseConfig):
    """
    Consulta os produtos contidos no banco semaforo juntamente com os dados do banco do ERP
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de produtos
    """
    # abre a connection com o banco
    db = database.Connection(db_config)
    connection = db.get_conect()
    # connection.start_transaction()
    cursor = connection.cursor()

    # obtem os dados do banco
    # logger.warning(query)
    cursor.execute(db_config.sql)
    columns = [col[0] for col in cursor.description]
    results = [dict(zip(columns, row)) for row in cursor.fetchall()]
    cursor.close()
    connection.close()

    produtos = []
    for result in results:
        produtos.append(Product(result))

    return produtos


def protocol_products(job_config_dict: dict, products: list, db_config: DatabaseConfig) -> None:
    """
    Protocola no banco semaforo os produtos que foram enviados para a api okvendas
    Args:
        job_config_dict: Configuração do job
        products: Lista de produtos enviados para a api okvendas
        db_config: Configuracao do banco de dados
    """
    try:
        if len(products) > 0:
            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()
            for prod in products:
                try:
                    dados_produto = [prod.codigo_erp, prod.preco_estoque[0].codigo_erp_atributo]
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Protocolando codigo_erp {dados_produto[0]} sku {dados_produto[1]}', 'info', 'PRODUTO')
                    cursor.execute(queries.get_product_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, dados_produto))
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Linhas afetadas {cursor.rowcount}', 'info', 'PRODUTO')
                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Erro ao protocolar produto {prod.codigo_erp}: {str(e)}', 'error', 'PRODUTO')
            cursor.close()
            connection.commit()
            connection.close()
    except Exception as e:
        raise e


def mount_list_photos(products: List[Product]):
    photos = {}

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in photos:

            length_photos = len(photos[key])

            if contains_photo(photos[key], product.imagem_base64):
                continue
            else:
                order_photo = length_photos + 1
                photo_sku = PhotosSku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{order_photo}', order_photo,
                                      False)
                photos[key].append(photo_sku)

        else:
            # Mount photo
            photo_sku = PhotosSku(product.imagem_base64, codigo_erp, f'{codigo_erp}_{1}', 1,
                                  True)

            photos[key] = [photo_sku]

    return photos


def remove_repeat_products(products: List[Product]):
    product_keys = {}
    result_products = []

    for product in products:
        # Grouping by codigo_erp and codigo_erp_variacao
        codigo_erp = product.codigo_erp
        codigo_erp_variacao = product.preco_estoque[0].codigo_erp_atributo

        key = (codigo_erp, codigo_erp_variacao)

        if key in product_keys:
            continue
        else:
            product_keys[key] = ""
            result_products.append(product)

    return result_products


def send_photos_products(photos_sku: dict, keys_photos_sent: dict, limit_photos_sent: int):
    count_success = 0
    list_photos = []
    for photo_key in photos_sku:

        if photo_key in keys_photos_sent:
            list_photo = photos_sku[photo_key]

            for photo in list_photo:
                if (len(list_photos) + 1) <= limit_photos_sent:
                    list_photos.append(photo)

                if len(list_photos) == limit_photos_sent:
                    success = api_okvendas.put_photos_sku(list_photos)
                    if not success:
                        return success
                    else:
                        count_success = count_success + 1
                        list_photos = []

    if 0 < len(list_photos) <= 50:
        return api_okvendas.put_photos_sku(list_photos)

    return count_success > 0 if True else False


def contains_photo(photos: List[PhotosSku], imagem_base64: str):
    for photo in photos:
        if photo.base64_foto == imagem_base64:
            return True
    return False


def job_product_tax(job_config_dict: dict):
    with lock:
        """
        Job para inserir o imposto do produto com as listas de imposto no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir a relação de imposto de produtos no semaforo nao encontrado', 'warning', 'IMPOSTO')
            return

        try:
            db_product_tax = query_list_products_tax(job_config_dict, db_config)
            if not insert_update_semaphore_product_tax(job_config_dict, db_config, db_product_tax):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Nao foi possivel inserir os impostos de produtos no banco semaforo', 'error', 'IMPOSTO')
                return

            api_productstax = []
            for prod in db_product_tax:
                api_productstax.append(ImpostoProduto(
                    prod.sku_code
                    , prod.ncm
                    , prod.taxation_group
                    , prod.customer_group
                    , prod.origin_uf
                    , prod.destination_uf
                    , prod.mva_iva
                    , prod.intrastate_icms
                    , prod.interstate_icms
                    , prod.percentage_reduction_base_calculation
                    , prod.identifier
                ))

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Enviando impostos via api okvendas', 'info', 'IMPOSTO')
            total = len(api_productstax)
            page = 100
            limit = 100 if total > 100 else total
            offset = 0

            partial_taxes = api_productstax[offset:limit]
            while limit <= total:
                response = api_okvendas.post_product_tax(partial_taxes)

                for res in response:
                    identificador = [i.identificador for i in api_productstax if i.identificador == res.identifiers[0]][0]
                    if res.status == 1:
                        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,f'Imposto Cadastrado/Atualizado com sucesso - Identificador: {identificador}','warning', 'IMPOSTO')
                        if protocol_semaphore_product_tax(job_config_dict, db_config, identificador):
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Imposto de Produtos {identificador} protocolado no banco semaforo', 'info', 'IMPOSTO')
                        else:
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar o imposto do produto {identificador}', 'warning', 'IMPOSTO')

                limit = limit + page
                offset = offset + page
                partial_taxes = api_productstax[offset:limit]

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro {str(ex)}', 'error', 'IMPOSTO')

def job_product_tax_full(job_config_dict: dict):
    with lock:
        """
        Job para inserir os impostos dos produtos em lote no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir a relação de impostos dos produtos em lote no semaforo nao encontrado', 'warning', 'IMPOSTO_LOTE')
            return

        try:
            db_product_tax = query_list_products_tax(job_config_dict, db_config)
            if not insert_update_semaphore_product_tax(job_config_dict, db_config, db_product_tax):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Nao foi possivel inserir os impostos dos produtos em lote no banco semaforo', 'error', 'IMPOSTO_LOTE')
                return

            # Define a lista de dicionários para construir o DataFrame
            data = []
            for product_tax in db_product_tax:
                data.append({
                    'codigo_sku': product_tax.sku_code,
                    'ncm': product_tax.ncm,
                    'grupo_tributacao': product_tax.taxation_group,
                    'grupo_cliente': product_tax.customer_group,
                    'uf_origem': product_tax.origin_uf,
                    'uf_destino': product_tax.destination_uf,
                    'mva_iva': None if product_tax.mva_iva is None else round(product_tax.mva_iva, 2), #arredonda para 2 casas decimais
                    'icms_intraestadual': None if product_tax.intrastate_icms is None else round(product_tax.intrastate_icms, 2), #arredonda para 2 casas decimais
                    'icms_interestadual': product_tax.interstate_icms,
                    'percentual_reducao_base_calculo': None if product_tax.percentage_reduction_base_calculation is None else round(product_tax.percentage_reduction_base_calculation, 2) #arredonda para 4 casas decimais
                    # arredonda para 4 casas decimais
                })

            # Cria um DataFrame a partir da lista de dicionários
            df = pd.DataFrame(data)

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Enviando impostos em lote via api okvendas', 'info', 'IMPOSTO_LOTE')
            response = api_okvendas.post_product_full_tax(df)

            if response['Status'] == 1:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,f'Impostos em lote Cadastrados/Atualizados com sucesso na Api OkVendas','warning', 'IMPOSTO_LOTE')
                if protocol_semaphore_product_tax_full(job_config_dict, db_config, db_product_tax):
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Impostos em lote protocolados com sucesso no banco semaforo', 'info', 'IMPOSTO_LOTE')
                else:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar os impostos dos produtos em lote no banco semaforo', 'warning', 'IMPOSTO_LOTE')
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'Falha ao Cadastrar/Atualizar os Impostos dos produtos em lote na Api OkVendas', 'warning', 'IMPOSTO_LOTE')

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro {str(ex)}', 'error', 'IMPOSTO_LOTE')

def query_list_products_tax(job_config_dict: dict, db_config: DatabaseConfig) -> List[ProductTax]:
    """
        Consultar no banco semáforo a lista de produtos relacionados a lista de preço

        Args:
            job_config_dict: Configuração do job
            db_config: Configuração do banco de dados

        Returns:
        Lista de produtos relacionados ao imposto informada
        """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            lists = [ProductTax(**p) for p in results]
            return lists

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar imposto do produto no banco semaforo: {str(ex)}', 'error', 'IMPOSTO')

    return []


def insert_update_semaphore_product_tax(job_config_dict: dict, db_config: DatabaseConfig, lists: List[ProductTax]) -> bool:
    """
    Insere os imposts no banco semáforo
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de impostos dos produtos

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.identifier, ' ', IntegrationType.IMPOSTO.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar listas de impostos no banco semaforo: {str(ex)}', 'error', 'IMPOSTO')

    return False

def protocol_semaphore_item(job_config_dict: dict, db_config: DatabaseConfig, identifier: str, identifier2: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        if identifier2 is None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        else:
            cursor.execute(queries.get_protocol_semaphore_id2_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier, identifier2]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0
    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar item no banco semaforo: {str(ex)}', 'error', 'IMPOSTO')


def protocol_semaphore_product_tax(job_config_dict: dict, db_config: DatabaseConfig, identifier: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        if identifier is not None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar imposto do produto no banco semaforo: {str(ex)}', 'error', 'IMPOSTO')

def protocol_semaphore_product_tax_full(job_config_dict: dict, db_config: DatabaseConfig, lists: List[ProductTax]) -> bool:
    """
    protocola os impostos em lote no banco semáforo
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de impostos dos produtos

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.identifier, IntegrationType.IMPOSTO.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_protocol_semaphore_id3_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar as listas de impostos em lote no banco semaforo: {str(ex)}', 'error', 'IMPOSTO_LOTE')

    return False

# job_product_tax



class Representative:
    def __init__(self, codigo_externo, nome, telefone_celular, login,
                 password, supervisor,ativar_representante, email,
                  rv_nome, rv_codigo_externo):
        self.sku_code: str = codigo_externo
        self.name: str = nome
        self.mobile_phone: str = telefone_celular
        self.login: str = login
        self.password: str = password
        self.supervisor: bool = supervisor
        self.representative_active: bool = ativar_representante
        self.email: str = email
        self.rv_name: str = rv_nome
        self.rv_sku_code: float = rv_codigo_externo